@@ -26,14 +26,13 @@ use std::time::Duration;
 
 use nucliadb_core::tracing::debug;
 use nucliadb_core::{node_error, NodeResult};
-use nucliadb_vectors::config::VectorConfig;
 
 use super::resource_cache::{CacheResult, ResourceCache, ResourceLoadGuard};
 use crate::disk_structure;
 use crate::errors::ShardNotFoundError;
 use crate::settings::Settings;
 use crate::shards::metadata::{ShardMetadata, ShardsMetadataManager};
-use crate::shards::writer::ShardWriter;
+use crate::shards::writer::{NewShard, ShardWriter};
 use crate::shards::ShardId;
 
 /// This cache allows the user to block shards, ensuring that they will not be loaded from disk.
@@ -122,10 +121,10 @@ impl ShardWriterCache {
         self.cache.lock().expect("Poisoned cache lock")
     }
 
-    pub fn create(&self, metadata: ShardMetadata, vector_config: VectorConfig) -> NodeResult<Arc<ShardWriter>> {
-        let shard_id = metadata.id();
-        let metadata = Arc::new(metadata);
-        let shard = Arc::new(ShardWriter::new(metadata.clone(), vector_config)?);
+    pub fn create(&self, new: NewShard) -> NodeResult<Arc<ShardWriter>> {
+        let shard_id = new.shard_id.clone();
+        let (shard, metadata) = ShardWriter::new(new, &self.shards_path)?;
+        let shard = Arc::new(shard);
 
         self.metadata_manager.add_metadata(metadata);
         self.cache().add_active_shard(&shard_id, &shard);
@@ -231,6 +230,7 @@ impl ShardWriterCache {
 
 #[cfg(test)]
 mod tests {
+    use std::collections::HashMap;
     use std::fs;
     use std::sync::Arc;
     use std::thread::sleep;
@@ -243,7 +243,9 @@ mod tests {
 
     use super::ShardWriterCache;
     use crate::settings::{EnvSettings, Settings};
+    use crate::shards::indexes::DEFAULT_VECTORS_INDEX_NAME;
     use crate::shards::metadata::ShardMetadata;
+    use crate::shards::writer::NewShard;
     use crate::shards::ShardId;
 
     #[test]
@@ -260,9 +262,14 @@ mod tests {
         let shard_0_path = settings.shards_path().join(shard_id_0.clone());
         fs::create_dir(settings.shards_path()).unwrap();
 
-        let shard_meta =
-            ShardMetadata::new(shard_0_path.clone(), shard_id_0.clone(), "kbid".to_string(), Channel::EXPERIMENTAL);
-        cache.create(shard_meta, VectorConfig::default()).unwrap();
+        cache
+            .create(NewShard {
+                kbid: "kbid".to_string(),
+                shard_id: shard_id_0.clone(),
+                channel: Channel::EXPERIMENTAL,
+                vector_configs: HashMap::from([(DEFAULT_VECTORS_INDEX_NAME.to_string(), VectorConfig::default())]),
+            })
+            .unwrap();
 
         let shard_0 = cache.get(&shard_id_0).unwrap();
 
@@ -301,9 +308,14 @@ mod tests {
         assert!(cache.get(&shard_id_0).is_err());
 
         // Recreating the shard should work (i.e: it's not stuck in the deletion state)
-        let shard_meta =
-            ShardMetadata::new(shard_0_path.clone(), shard_id_0.clone(), "kbid".to_string(), Channel::EXPERIMENTAL);
-        cache.create(shard_meta, VectorConfig::default()).unwrap();
+        cache
+            .create(NewShard {
+                kbid: "kbid".to_string(),
+                shard_id: shard_id_0.clone(),
+                channel: Channel::EXPERIMENTAL,
+                vector_configs: HashMap::from([(DEFAULT_VECTORS_INDEX_NAME.to_string(), VectorConfig::default())]),
+            })
+            .unwrap();
 
         assert!(cache.get(&shard_id_0).is_ok());
     }
