@@ -201,10 +201,16 @@ impl ShardReader {
         let info = info_span!(parent: &span, "vector count");
         let vector_task = || {
             run_with_telemetry(info, || {
-                read_rw_lock(&self.vector_readers)
-                    .get(DEFAULT_VECTORS_INDEX_NAME)
-                    .expect("Default vectors index should never be deleted (yet)")
-                    .count()
+                let vector_readers = read_rw_lock(&self.vector_readers);
+                if let Some(reader) = vector_readers.get(DEFAULT_VECTORS_INDEX_NAME) {
+                    return reader.count();
+                }
+
+                let mut count = 0;
+                for reader in vector_readers.values() {
+                    count += reader.count()?;
+                }
+                Ok(count)
             })
         };
 
@@ -243,11 +249,12 @@ impl ShardReader {
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn get_vectors_keys(&self) -> NodeResult<Vec<String>> {
-        read_rw_lock(&self.vector_readers)
-            .get(DEFAULT_VECTORS_INDEX_NAME)
-            .expect("Default vectors index should never be deleted (yet)")
-            .stored_ids()
+    pub fn get_vectors_keys(&self, vectorset_id: &str) -> NodeResult<Vec<String>> {
+        if let Some(reader) = read_rw_lock(&self.vector_readers).get(vectorset_id) {
+            reader.stored_ids()
+        } else {
+            Err(node_error!("Vectorset {vectorset_id} does not exist"))
+        }
     }
 
     #[tracing::instrument(skip_all)]
@@ -372,7 +379,8 @@ impl ShardReader {
                 let prefilter = PreFilterRequest {
                     timestamp_filters: vec![],
                     security: None,
-                    formula: Some(BooleanExpression::Operation(op)),
+                    labels_formula: Some(BooleanExpression::Operation(op)),
+                    keywords_formula: None,
                 };
 
                 let prefiltered = read_rw_lock(&self.text_reader).prefilter(&prefilter)?;
@@ -469,7 +477,6 @@ impl ShardReader {
     #[tracing::instrument(skip_all)]
     pub fn search(&self, search_request: SearchRequest) -> NodeResult<SearchResponse> {
         let query_plan = query_planner::build_query_plan(self.versions.paragraphs, search_request)?;
-
         let search_id = uuid::Uuid::new_v4().to_string();
         let span = tracing::Span::current();
         let mut index_queries = query_plan.index_queries;
@@ -673,15 +680,26 @@ impl ShardReader {
     ) -> NodeResult<VectorSearchResponse> {
         let vectorset = &request.vector_set;
         if vectorset.is_empty() {
-            read_rw_lock(&self.vector_readers)
-                .get(DEFAULT_VECTORS_INDEX_NAME)
-                .expect("Default vectors index should never be deleted (yet)")
-                .search(request, context)
+            let vector_readers = read_rw_lock(&self.vector_readers);
+            if let Some(reader) = vector_readers.get(DEFAULT_VECTORS_INDEX_NAME) {
+                reader.search(request, context)
+            } else if vector_readers.len() == 1 {
+                // no default vectorset but only one exist, consider it the
+                // default
+                let reader = vector_readers.values().next().unwrap();
+                reader.search(request, context)
+            } else {
+                Err(node_error!("Query without vectorset but shard has multiple vector indexes"))
+            }
         } else {
             let vector_readers = read_rw_lock(&self.vector_readers);
             let reader = vector_readers.get(vectorset);
             if let Some(reader) = reader {
                 reader.search(request, context)
+            } else if vector_readers.len() == 1 && vector_readers.contains_key(DEFAULT_VECTORS_INDEX_NAME) {
+                // Only one vectorset with default name, use it!
+                // We can remove this once all vectorsets are named (there are no default vectorsets)
+                vector_readers.get(DEFAULT_VECTORS_INDEX_NAME).unwrap().search(request, context)
             } else {
                 Err(node_error!("Vectorset '{vectorset}' not found"))
             }
