@@ -86,10 +86,20 @@ impl ParagraphWriter for ParagraphWriterService {
         let v = time.elapsed().as_millis();
         debug!("{id:?} - Processing paragraphs to delete: starts at {v} ms");
 
-        for paragraph_id in &resource.paragraphs_to_delete {
-            let uuid_term = Term::from_field_text(self.schema.paragraph, paragraph_id);
-            self.writer.delete_term(uuid_term);
+        // Delete all paragraphs matching the field_uuid
+        //
+        // Bw/c delete by paragraph_id
+        for field_or_paragraph_id in &resource.paragraphs_to_delete {
+            let field_uuid_term = Term::from_field_text(self.schema.field_uuid, field_or_paragraph_id);
+            self.writer.delete_term(field_uuid_term);
+            // TODO: remove deletion by paragraph_id when deletion by field gets
+            // promoted
+            let paragraph_uuid_term = Term::from_field_text(self.schema.paragraph, field_or_paragraph_id);
+            self.writer.delete_term(paragraph_uuid_term);
         }
+
+        self.writer.commit()?;
+
         let v = time.elapsed().as_millis();
         debug!("{id:?} - Processing paragraphs to delete: ends at {v} ms");
 
@@ -274,7 +284,8 @@ impl ParagraphWriterService {
                 doc.add_u64(self.schema.end_pos, end_pos);
                 doc.add_u64(self.schema.index, index);
                 doc.add_text(self.schema.split, split);
-                doc.add_text(self.schema.field_uuid, format!("{}/{}", resource.resource.as_ref().unwrap().uuid, field));
+                let field_uuid = format!("{}/{}", resource.resource.as_ref().unwrap().uuid, field);
+                doc.add_text(self.schema.field_uuid, field_uuid.clone());
 
                 self.writer.delete_term(paragraph_term);
                 self.writer.add_document(doc)?;
