@@ -21,6 +21,8 @@
 use std::fs::File;
 use std::io::{self, BufWriter, Seek, SeekFrom, Write};
 
+use crate::config::{VectorConfig, VectorType};
+
 use super::usize_utils::*;
 
 // A data store schema.
@@ -68,11 +70,12 @@ pub trait Interpreter {
 }
 
 pub trait IntoBuffer {
-    fn serialize_into<W: io::Write>(self, w: W) -> io::Result<()>;
+    fn serialize_into<W: io::Write>(self, w: W, vector_type: &VectorType) -> io::Result<()>;
 }
 
+#[cfg(test)]
 impl<T: AsRef<[u8]>> IntoBuffer for T {
-    fn serialize_into<W: io::Write>(self, mut w: W) -> io::Result<()> {
+    fn serialize_into<W: io::Write>(self, mut w: W, _: &VectorType) -> io::Result<()> {
         w.write_all(self.as_ref())
     }
 }
@@ -93,7 +96,7 @@ use lazy_static::lazy_static;
 use libc;
 
 #[cfg(not(target_os = "windows"))]
-pub fn will_need(src: &[u8], id: usize, dimension: usize) {
+pub fn will_need(src: &[u8], id: usize, vector_len: usize) {
     lazy_static! {
         static ref PAGE_SIZE: usize = unsafe { libc::sysconf(libc::_SC_PAGESIZE) } as usize;
     };
@@ -102,7 +105,6 @@ pub fn will_need(src: &[u8], id: usize, dimension: usize) {
     // Will only load up to the vectors section, it ignores the key/labels because
     // they are harder to estimate and less useful (only when filtering, similarity is always used)
     let metadata_len = 16; // Estimate
-    let vector_len = dimension * 4 + 4;
     let node_size = HEADER_LEN + metadata_len + vector_len;
 
     // Align node pointer to the start page, as required by madvise
@@ -118,9 +120,13 @@ pub fn will_need(src: &[u8], id: usize, dimension: usize) {
 }
 
 #[cfg(target_os = "windows")]
-pub fn will_need(src: &[u8], id: usize, dimension: usize) {}
+pub fn will_need(src: &[u8], id: usize, vector_len: usize) {}
 
-pub fn create_key_value<D: IntoBuffer>(recipient: &mut File, slots: Vec<D>) -> io::Result<()> {
+pub fn create_key_value<D: IntoBuffer>(
+    recipient: &mut File,
+    slots: Vec<D>,
+    vector_type: &VectorType,
+) -> io::Result<()> {
     let fixed_size = (HEADER_LEN + (POINTER_LEN * slots.len())) as u64;
     recipient.set_len(fixed_size)?;
 
@@ -134,10 +140,16 @@ pub fn create_key_value<D: IntoBuffer>(recipient: &mut File, slots: Vec<D>) -> i
     // Serializing values into the recipient. Each slot is serialized at the end of the
     // loop and its address pointer is written in the pointer section.
     let mut pointer_section_cursor = HEADER_LEN as u64;
+    let alignment = vector_type.vector_alignment();
     for slot in slots {
         // slot serialization
-        let slot_address = recipient_buffer.seek(SeekFrom::End(0))?;
-        slot.serialize_into(&mut recipient_buffer)?;
+        let mut slot_address = recipient_buffer.seek(SeekFrom::End(0))?;
+        if slot_address as usize % alignment > 0 {
+            let pad = alignment - (slot_address as usize % alignment);
+            recipient_buffer.seek(SeekFrom::Current(pad as i64))?;
+            slot_address += pad as u64;
+        }
+        slot.serialize_into(&mut recipient_buffer, vector_type)?;
 
         // The slot address needs to be written in the pointer section
         recipient_buffer.seek(SeekFrom::Start(pointer_section_cursor))?;
@@ -152,7 +164,11 @@ pub fn create_key_value<D: IntoBuffer>(recipient: &mut File, slots: Vec<D>) -> i
 }
 
 // Merge algorithm. Returns the number of elements merged into the file.
-pub fn merge<S: Interpreter + Copy>(recipient: &mut File, producers: &[(S, &[u8])]) -> io::Result<bool> {
+pub fn merge<S: Interpreter + Copy>(
+    recipient: &mut File,
+    producers: &[(S, &[u8])],
+    config: &VectorConfig,
+) -> io::Result<bool> {
     // Number of elements, deleted or alive.
     let mut prologue_section_size = HEADER_LEN;
     // To know the range of valid ids per producer
@@ -184,6 +200,7 @@ pub fn merge<S: Interpreter + Copy>(recipient: &mut File, producers: &[(S, &[u8]
     let mut id_section_cursor = HEADER_LEN;
     let mut has_deletions = false;
 
+    let alignment = config.vector_type.vector_alignment();
     while producer_cursor < producers.len() {
         // If the end of the current producer was reached we move
         // to the start of the next producer.
@@ -201,7 +218,12 @@ pub fn merge<S: Interpreter + Copy>(recipient: &mut File, producers: &[(S, &[u8]
         if interpreter.keep_in_merge(element_slice) {
             // Moving to the end of the file to write the current element.
             let (exact_element, _) = interpreter.read_exact(element_slice);
-            let element_pointer = recipient_buffer.seek(SeekFrom::End(0))?;
+            let mut element_pointer = recipient_buffer.seek(SeekFrom::End(0))?;
+            if element_pointer as usize % alignment > 0 {
+                let pad = alignment - (element_pointer as usize % alignment);
+                recipient_buffer.seek(SeekFrom::Current(pad as i64))?;
+                element_pointer += pad as u64;
+            }
             recipient_buffer.write_all(exact_element)?;
             // Moving to the next free slot in the section id to write
             // the offset where the element was written.
@@ -263,7 +285,7 @@ mod tests {
         let elems: [u32; 5] = [0, 1, 2, 3, 4];
         let expected: Vec<_> = elems.iter().map(|x| x.to_be_bytes()).collect();
         let mut buf = tempfile::tempfile().unwrap();
-        create_key_value(&mut buf, expected.clone()).unwrap();
+        create_key_value(&mut buf, expected.clone(), &VectorType::DenseF32Unaligned).unwrap();
 
         let buf_map = unsafe { memmap2::Mmap::map(&buf).unwrap() };
         let no_values = stored_elements(&buf_map);
@@ -288,9 +310,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -299,7 +321,7 @@ mod tests {
         let mut merge_store = tempfile::tempfile().unwrap();
         let elems = vec![(TElem, v0_map.as_ref()), (TElem, v1_map.as_ref()), (TElem, v2_map.as_ref())];
 
-        merge(&mut merge_store, &elems).unwrap();
+        merge(&mut merge_store, &elems, &VectorConfig::default()).unwrap();
         let merge_map = unsafe { memmap2::Mmap::map(&merge_store).unwrap() };
         let number_of_elements = stored_elements(&merge_map);
         let values: Vec<u32> = (0..number_of_elements)
@@ -322,9 +344,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -341,7 +363,7 @@ mod tests {
             (interpreter, v2_map.as_ref()),
         ];
 
-        merge::<(GreaterThan, TElem)>(&mut merge_store, elems.as_slice()).unwrap();
+        merge::<(GreaterThan, TElem)>(&mut merge_store, elems.as_slice(), &VectorConfig::default()).unwrap();
         let expected: Vec<u32> = vec![2, 3, 4, 5, 6, 7, 8, 9, 10, 11];
         let merge_map = unsafe { memmap2::Mmap::map(&merge_store).unwrap() };
         let number_of_elements = stored_elements(&merge_map);
@@ -364,9 +386,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -383,7 +405,7 @@ mod tests {
             (interpreter, v2_map.as_ref()),
         ];
 
-        merge::<(GreaterThan, TElem)>(&mut merge_store, elems.as_slice()).unwrap();
+        merge::<(GreaterThan, TElem)>(&mut merge_store, elems.as_slice(), &VectorConfig::default()).unwrap();
         let expected: Vec<u32> = vec![2, 3, 4, 5, 6, 7, 8];
         let merge_map = unsafe { memmap2::Mmap::map(&merge_store).unwrap() };
         let number_of_elements = stored_elements(&merge_map);
@@ -406,9 +428,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -424,7 +446,7 @@ mod tests {
             ((GreaterThan(SIX), TElem), v2_map.as_ref()),
         ];
 
-        merge::<(GreaterThan, TElem)>(&mut merge_store, elems.as_slice()).unwrap();
+        merge::<(GreaterThan, TElem)>(&mut merge_store, elems.as_slice(), &VectorConfig::default()).unwrap();
         let expected: Vec<u32> = vec![1, 2, 4, 5, 7, 8];
         let merge_map = unsafe { memmap2::Mmap::map(&merge_store).unwrap() };
         let number_of_elements = stored_elements(&merge_map);
@@ -447,9 +469,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -466,7 +488,7 @@ mod tests {
             (interpreter, v2_map.as_ref()),
         ];
 
-        merge::<(GreaterThan, TElem)>(&mut merge_storage, elems.as_slice()).unwrap();
+        merge::<(GreaterThan, TElem)>(&mut merge_storage, elems.as_slice(), &VectorConfig::default()).unwrap();
         let expected: Vec<u32> = vec![3, 4, 5, 6, 7, 8];
         let merge_store = unsafe { memmap2::Mmap::map(&merge_storage).unwrap() };
         let number_of_elements = stored_elements(&merge_store);
@@ -489,9 +511,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -509,7 +531,7 @@ mod tests {
             (greater_than_10, v2_map.as_ref()),
         ];
 
-        merge::<(GreaterThan, TElem)>(&mut merge_storage, elems.as_slice()).unwrap();
+        merge::<(GreaterThan, TElem)>(&mut merge_storage, elems.as_slice(), &VectorConfig::default()).unwrap();
         let expected: Vec<u32> = vec![3, 4, 5];
         let merge_store = unsafe { memmap2::Mmap::map(&merge_storage).unwrap() };
         let number_of_elements = stored_elements(&merge_store);
@@ -532,9 +554,9 @@ mod tests {
         let mut v1_store = tempfile::tempfile().unwrap();
         let mut v2_store = tempfile::tempfile().unwrap();
 
-        create_key_value(&mut v0_store, v0).unwrap();
-        create_key_value(&mut v1_store, v1).unwrap();
-        create_key_value(&mut v2_store, v2).unwrap();
+        create_key_value(&mut v0_store, v0, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v1_store, v1, &VectorType::DenseF32Unaligned).unwrap();
+        create_key_value(&mut v2_store, v2, &VectorType::DenseF32Unaligned).unwrap();
 
         let v0_map = unsafe { memmap2::Mmap::map(&v0_store).unwrap() };
         let v1_map = unsafe { memmap2::Mmap::map(&v1_store).unwrap() };
@@ -552,7 +574,7 @@ mod tests {
             (interpreter, v2_map.as_ref()),
         ];
 
-        merge::<(GreaterThan, TElem)>(&mut file, elems.as_slice()).unwrap();
+        merge::<(GreaterThan, TElem)>(&mut file, elems.as_slice(), &VectorConfig::default()).unwrap();
         let expected: Vec<u32> = vec![];
         let merge_store = unsafe { memmap2::Mmap::map(&file).unwrap() };
         let number_of_elements = stored_elements(&merge_store);
