@@ -93,6 +93,20 @@ impl VectorReader for VectorReaderService {
             formula.extend(clause_labels);
         }
 
+        if !request.field_filters.is_empty() {
+            let mut field_atoms: Vec<Clause> = vec![];
+            for field in request.field_filters.iter() {
+                // split "a/title" into "a" and "title"
+                let parts = field.split('/').collect::<Vec<&str>>();
+                let field_type = parts.first().unwrap_or(&"").to_string();
+                let field_name = parts.get(1).unwrap_or(&"").to_string();
+                let atom_clause = Clause::Atom(AtomClause::key_field(field_type, field_name));
+                field_atoms.push(atom_clause);
+            }
+            let compound = CompoundClause::new(BooleanOperator::Or, field_atoms);
+            formula.extend(compound);
+        }
+
         if let Some(filter) = context.filtering_formula.as_ref() {
             let clause = query_io::map_expression(filter);
             formula.extend(clause);
