@@ -16,12 +16,13 @@ setup(
     zip_safe=True,
     include_package_data=True,
     package_data={"": ["*.txt", "*.rst"], "nucliadb_protos": ["py.typed"]},
-    packages=find_packages(),
+    packages=find_packages("src"),
+    package_dir={"": "src"},
     install_requires=[
         "protobuf >= 4.22.3, < 5",
         "grpcio >= 1.44.0",
         "grpcio-tools >= 1.44.0",
         "mypy-protobuf >= 3.4.0",
-        "types-protobuf >= 4.24, < 5"
+        "types-protobuf >= 4.24, < 5",
     ],
 )
